package com.bleujin.framework.configuration;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.Hashtable;

import org.xml.sax.InputSource;

/**
 * <pre>
 * Configuration  Ѵ.
 * 
 * ConfigurationFactory  Configuration νϽ Ѵ.
 * ConfiguartionFactory  newInstance( String instanceKeyName )  Ǹ KeyName   ְ  factory 
 *  ȴ. ѹ factory  ٽ  ʰ ̹̻Ǿִ νϽ ش.
 * 
 * ConfigurationFactory ϱ
 * 
 *      ConfigurationFactory factory = ConfigurationFactory.newInstance( keyname );
 *      factory.build( configurationFileName );  // xml  о̱
 * 
 * 
 * 
 * 
 * </pre>
 * 
 * @author Choi sei hwan <a href="mailto:sehan@i-on.net">Choi sei hwan</a>
 * @version 1.0
 */

public abstract class ConfigurationFactory {
	//  ConfigurationFactory νϽ ϴ hashtable
	private static Hashtable configurationFactoryCache = new Hashtable();

	// factory ̸
	protected String instanceKeyName = null;

	// protected ConfigurationType configurationType = null;

	//  ǰ ִ concrete ConfigurationFactory Ŭ  Ű
	private static ConfigurationFactoryType configurationFactoryType = ConfigurationFactoryType.DEFAULT;

	/**
	 * ConfigurationFactory νϽ ´. Singleton  νϽ ´.
	 * 
	 * @param instanceKeyName
	 *            String  νϽ ̸
	 * @throws ConfigurationException
	 * @return ConfigurationFactory
	 */
	public static ConfigurationFactory getInstance(String instanceKeyName) throws ConfigurationException {
		// ̹ Ǿִ νϽ ִ ãƺ.
		ConfigurationFactory instance = (ConfigurationFactory) configurationFactoryCache.get(instanceKeyName);

		if (instance == null) {
			// ã Ͽ  Ѵ.
			try {
				Constructor constructor = Class.forName(configurationFactoryType.getClassName()).getConstructor(new Class[] { String.class });
				instance = (ConfigurationFactory) (constructor.newInstance(new Object[] { instanceKeyName }));
			} catch (Exception ex) {
				throw new ConfigurationException("ConfigurationFactory instance not Create : " + ex.getLocalizedMessage(), ex);
			}

			//   νϽ Ѵ.
			configurationFactoryCache.put(instanceKeyName, instance);
		}

		return instance;
	}

	@Deprecated
	public static ConfigurationFactory createInstance(String instanceKeyName) throws ConfigurationException, SecurityException, NoSuchMethodException, ClassNotFoundException, IllegalArgumentException, InstantiationException, IllegalAccessException, InvocationTargetException {
		Constructor constructor = Class.forName(configurationFactoryType.getClassName()).getConstructor(new Class[] { String.class });
		ConfigurationFactory instance = (ConfigurationFactory) (constructor.newInstance(new Object[] { instanceKeyName }));
		return instance ;
	}

	
	/**
	 * filename  شϴ xml config  оδ.
	 * 
	 * @param filename
	 *            String ȯ漳  XML ϰ
	 * @throws ConfigurationBuildException
	 */
	public void build(String filename) throws ConfigurationBuildException {
		build(new File(filename));
	}

	/**
	 * File Class  ؼ xml config  оδ.
	 * 
	 * @param configFile
	 *            File ȯ漳  FILE Class
	 * @throws ConfigurationBuildException
	 */
	public void build(File configFile) throws ConfigurationBuildException {
		try {
			File cFile = null;
			cFile = configFile.getCanonicalFile();
			FileInputStream fis = new FileInputStream(cFile);
			build(fis);
		} catch (IOException ex) {
			throw new ConfigurationBuildException("file not found : " + configFile.getName(), ex);
		}
	}

	/**
	 * InputStream  ؼ xml config  еδ.
	 * 
	 * @param inputstream
	 *            InputStream ȯ漳  InputStream Class
	 * @throws ConfigurationBuildException
	 */
	public void build(InputStream inputstream) throws ConfigurationBuildException {
		build(new InputSource(inputstream));
	}

	/**
	 * inputsource  ؼ xml config  оδ.
	 * 
	 * @param inputsource
	 *            InputSource ȯ漳  InputSource Class
	 * @throws ConfigurationBuildException
	 */
	public abstract void build(InputSource inputsource) throws ConfigurationBuildException;

	/**
	 * elementPathString شϴ Configuration νϽ ´.
	 * 
	 * @param elementPathString
	 *            String  Configuration νϽ path ڿ
	 * @throws NotFoundXmlTagException
	 * @throws NotBuildException
	 * @throws ConfigurationException
	 * @return Configuration
	 */
	public abstract Configuration getConfiguration(String elementPathString) throws NotFoundXmlTagException, NotBuildException, ConfigurationException;

	/**
	 * elementPathString شϴ Configuration νϽ ´.
	 * 
	 * @param elementPathString
	 *            String  Configuration νϽ path ڿ
	 * @throws NotFoundXmlTagException
	 * @throws NotBuildException
	 * @throws ConfigurationException
	 * @return Configuration[]
	 */
	public abstract Configuration[] getConfigurations(String elementPathString) throws NotFoundXmlTagException, NotBuildException, ConfigurationException;

	// /**
	// *  Factory  Ŭ̸ ϱ
	// *
	// * @param className - Factory Ŭ 
	// */
	// public static void setClassName( String className )
	// {
	// ConfigurationFactory.className = className;
	// }
	//
	// /**
	// *  Factory  Ŭ̸ 
	// *
	// * @return  Factory  Ŭ̸
	// */
	// public static String getClassName()
	// {
	// return className;
	// }

	/**
	 *  Factory ϴ Configuration  Type ϱ
	 * 
	 * @param configType
	 */
	// public void setConfigurationType( ConfigurationType configType )
	// {
	// this.configurationType = configType;
	// }
	/**
	 *  Factory ϴ Configuration  Type 
	 * 
	 * @return ConfiguraitonType. ⺻ Configuration.DEFAULT
	 */
	// public ConfigurationType getConfigurationType()
	// {
	// return configurationType;
	// }
	/**
	 * ConfigurationFactory  InstanceKey ̸ ´.
	 * 
	 * @return String
	 */
	public String getInstanceKeyName() {
		return instanceKeyName;
	}

	public ConfigurationFactoryType getConfigurationFactoryType() {
		return configurationFactoryType;
	}

	public void setConfigurationFactoryType(ConfigurationFactoryType configurationFactoryType) {
		this.configurationFactoryType = configurationFactoryType;
	}

	protected static void removeConfiguration(String instanceKeyName) {
		configurationFactoryCache.remove(instanceKeyName);
	}

}
