package com.bleujin.framework.db;

import java.io.IOException;
import java.io.StringWriter;
import java.io.Writer;
import java.net.URL;
import java.sql.Array;
import java.sql.Blob;
import java.sql.Clob;
import java.sql.Ref;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Types;

import com.bleujin.framework.db.bean.ResultSetHandler;
import com.bleujin.framework.db.procedure.IQueryable;
import com.bleujin.framework.db.rowset.WebRowSet;
import com.bleujin.framework.util.StringUtil;

/**
 * CLOB precessing and Adapter Pattern
 * <p>Title: Database framework</p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: I-ON Communications</p>
 * @author bleujin
 * @version 1.0
 */

public class RowsImpl extends WebRowSet implements Rows{

    private Rows nextRows = null;
    private IQueryable query ;
    protected RowsImpl(IQueryable query) throws SQLException {
        super();
        this.query = query ;
        setXmlWriter(new IONXmlWriter());
    }

    public String getString(int i) throws SQLException {
        ResultSetMetaData meta = this.getMetaData();
        if (meta.getColumnType(i) == Types.CLOB) {
            if (this.getClob(i) == null) return StringUtil.EMPTY;
            else {
                return this.getClobString(i) ;
            }
//            else return ClobToString2( ((OracleResultSet)this.getOriginalRow()).getCLOB(i) ) ;
        } else {
            return super.getString(i);
        }
    }

    protected String getClobString(int i) throws SQLException {
        return RowsUtils.clobToString(this.getClob(i)) ;
    }

    public String getString(String column) throws SQLException {
        return getString(getColumnIndex(column));
    }

    public Object getObject(int i) throws SQLException{
        return getClobObject(i) ;
    }

    private Object getClobObject(int i) throws SQLException{
        ResultSetMetaData meta = this.getMetaData();
        if (meta.getColumnType(i) == Types.CLOB) {
            if (this.getClob(i) == null) return StringUtil.EMPTY;
            else return RowsUtils.clobToString(this.getClob(i)) ;
        } else {
            return super.getObject(i);
        }
    }

    public Object getObject(String column) throws SQLException{
        return getClobObject(getColumnIndex(column)) ;
    }

    public boolean getBoolean(int i) throws SQLException {
        ResultSetMetaData meta = this.getMetaData();

        if (meta.getColumnType(i) == Types.VARCHAR || meta.getColumnType(i) == Types.CHAR) {
            if (super.getString(i) == null) return false;
            return ( StringUtil.T.equals(super.getString(i).toUpperCase()) || StringUtil.TRUE.equals(super.getString(i).toUpperCase()) ) ? true : false;
        } else {
            return super.getBoolean(i);
        }
    }

    public boolean getBoolean(String column) throws SQLException {
        return getBoolean(getColumnIndex(column));
    }

    private int getColumnIndex(String column) throws SQLException {
        ResultSetMetaData meta = this.getMetaData();
        for (int i = 1, last = meta.getColumnCount(); i <= last; i++) {
            if (meta.getColumnName(i).toUpperCase().equals(column.toUpperCase())) {
                return i;
            }
        }
        throw new SQLException("Not Valid Column Name. " + column + " Not Exists");
    }

//    private Map nameToIndexMap = null;
    protected String getClobString(String column) throws SQLException {
        int i = getColumnIndex(column);

        return this.getClobString(i);
    }

    public int getRowCount() {
        try {
            int result = 0;
            int currLoc = super.getRow();
            if(!last()) {
                return 0;
            }
            result = super.getRow();

            if(currLoc > 0) {
                absolute(currLoc);
            } else {
                beforeFirst();
            }
            return result;
        } catch(SQLException ex) {
            throw RepositoryException.throwIt(ex) ;
        }
    }

	public Rows setNextRows(Rows rows){
    	this.nextRows = rows ;
        return rows ;
    }

    public Rows getNextRows(){
    	return nextRows ;
    }

    public void setXmlWriter(IXmlWriter writer) throws SQLException {
        super.setXmlWriter(writer);
    }

    public void writeXml(Writer writer, IXmlWriter xmlWriter) throws SQLException {
        if (xmlWriter != null) {
            setXmlWriter(xmlWriter);
        }
        writeXml(writer);
    }

    public IXmlWriter getIXmlWriter() throws SQLException {
        return (IXmlWriter)super.getXmlWriter() ;
    }


    public String toString() {
        Writer writer = new StringWriter();
        try {
            this.writeXml(writer);
            writer.flush();
            writer.close();
        } catch (IOException ex) {
            ex.printStackTrace();
        } catch (SQLException ex) {
            ex.printStackTrace();
        }
        return writer.toString();
    }

    public URL getURL(int columnIndex) {
        throw new  NoSuchMethodError() ;
    }

    public void updateArray(int columnIndex, Array x) {
        throw new  NoSuchMethodError() ;
    }

    public void updateBlob(int columnIndex, Blob x) {
        throw new  NoSuchMethodError() ;
    }

    public void updateClob(int columnIndex, Clob x) {
        throw new  NoSuchMethodError() ;
    }

    public void updateRef(int columnIndex, Ref x) {
        throw new  NoSuchMethodError() ;
    }

    public URL getURL(String columnName) {
        throw new  NoSuchMethodError() ;
    }

    public void updateArray(String columnName, Array x) {
        throw new  NoSuchMethodError() ;
    }

    public void updateBlob(String columnName, Blob x) {
        throw new  NoSuchMethodError() ;
    }

    public void updateClob(String columnName, Clob x) {
        throw new  NoSuchMethodError() ;
    }

    public void updateRef(String columnName, Ref x) {
        throw new  NoSuchMethodError() ;
    }

    public String getDefaultString(String col1, String defaultString) throws SQLException {
        String str = this.getString(col1) ;
        return (str == null || "".equals(str)) ? defaultString : col1  ;
    }

	public Row firstRow() {
		return ExtendTransformer.fetchFirstToRow(this) ;
    }

	
	public Rows nextPageRows() throws SQLException{
		Page nextPage = query.getPage().getNextPage() ; 
		query.setPage(nextPage) ;
		return query.execQuery() ;
	}
	
	public Rows prePageRows() throws SQLException {
		query.setPage(query.getPage().getPrePage()) ;
		return query.execQuery() ;
		
	}
	
	public Object toHandle(ResultSetHandler rsh) throws SQLException {
		return rsh.handle(this);
	}

}


