package com.bleujin.framework.db.manager;

import java.sql.Connection;
import java.sql.SQLException;

import com.bleujin.framework.db.IDBController;
import com.bleujin.framework.db.procedure.RepositoryService;
import com.bleujin.framework.util.Debug;


/**
 * Used Pattern : Templet Method Pattern
 * <p>Title: </p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: I-ON Communications</p>
 * @author bleujin
 * @version 1.0
 */
public abstract class DBManager {
    final static int DEFAULT_FETCHSIZE = 10;
    private boolean isCreate = false;

    private IDBController owner ;
    private String driverName = null;
    private String jdbcURL = null;
    private String userId = null;
    private String userPwd = null;

    public DBManager(){
        this("","","","") ;
    }

	public DBManager(String driverName, String jdbcURL, String userId, String userPwd) {
        this.driverName = driverName;
        this.jdbcURL = jdbcURL;
        this.userId = userId;
        this.userPwd = userPwd;
    }

    private void setCreated(boolean isCreate) {
        this.isCreate = isCreate;
    }

    private boolean isCreated() {
        return isCreate;
    }

    public abstract int getDBManagerType();
    public abstract String getDBType() ;
    public abstract RepositoryService getRepositoryService();

    public final synchronized void initPool(IDBController dc) throws SQLException {
    	if (isCreated()) return ;
    	if (owner == null) owner = dc ; // set Owner
    	
    	myInitPool() ;
    	setCreated(true) ;
		Debug.debug("Init Pool ", getClass().getName(), "owner:" + owner.getName(), isCreate) ;
    }
    
    protected abstract void myInitPool() throws SQLException ;

    public final synchronized void destroyPool(IDBController dc) throws SQLException{
    	try {
    		if (owner != dc) return ;
    		if (! isCreate) return ;
    		
			myDestroyPool() ;
			setCreated(false) ;
			Debug.debug("Destroy Pool ", "owner:" + owner.getName(), isCreate) ;
		} catch (Exception e) {
			throw new SQLException(e.getMessage()) ;
		} 
    }
    
    protected abstract void myDestroyPool() throws Exception  ;
    
    
    public abstract Connection getConnection() throws SQLException;

    public void freeConnection(Connection conn) throws SQLException {
        if (conn != null) {
            conn.setAutoCommit(true); // default.. auto commit
            conn.close();
            // System.out.println( "-- freeConnnection() --" );
        }
    }


    public String getJdbcURL() {
        return jdbcURL;
    }

    public String getDriverName() {
        return driverName;
    }

    public String getUserPwd() {
        return userPwd;
    }

    public String getUserId() {
        return userId;
    }

    protected void set(String driverName, String jdbcURL, String userId, String userPwd){
    	this.driverName = driverName ;
    	this.jdbcURL = jdbcURL ;
    	this.userId = userId ;
    	this.userPwd = userPwd ;
    }
    
}
