package com.bleujin.framework.db.procedure;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import com.bleujin.framework.db.IDBController;
import com.bleujin.framework.db.Page;
import com.bleujin.framework.db.Rows;
import com.bleujin.framework.db.manager.DBManager;
import com.bleujin.framework.util.Debug;

/**
 * <p>
 * Title:
 * </p>
 * <p>
 * Description:
 * </p>
 * <p>
 * Copyright: Copyright (c) 2004
 * </p>
 * <p>
 * Company:
 * </p>
 *
 * @author bleujin
 * @version 1.0
 */
public abstract class Queryable implements IQueryable{

    public final static int DEFAULT_FETCHSIZE = 10;
    public final static int DEFAULT_QUERY_TIMEOUT = 0 ;

    private final IDBController dc ;
    private final String procSQL;
    private final int queryType ;
    private Page page ;

    private volatile boolean requestCancle = false ;

    protected Queryable(IDBController dc, String procSQL, int queryType) {
    	this(dc, procSQL, queryType, Page.DEFAULT) ;
    }

    protected Queryable(IDBController dc, String procSQL, int queryType, Page page) {
    	this.dc = dc ;
    	this.procSQL = procSQL;
    	this.queryType = queryType ;
    	this.page = page ;
    }

    public final int getQueryType(){
        return queryType ;
    }
    
    public final Page getPage(){
    	return page ;
    }
    
    public final void setPage(Page page){
    	this.page = page ;
    }

    public final String getDBType(){
    	return dc.getDBManager().getDBType() ;
    }
    
    public synchronized void cancel() throws SQLException, InterruptedException {
        if (requestCancle) return ; // balk

        requestCancle = true ;
        try {
            Statement stmt = getStatement();
            stmt.cancel();
        } catch (NullPointerException ex) {
            throw new InterruptedException("can't canceled because of Query is not running, Maybe query is not initialized or is already completed");
        } catch (SQLException ex) {
            throw ex ;
        } finally{
            requestCancle = false ;
        }
    }

    
    protected abstract Statement getStatement() ;
    protected abstract Rows myQuery(Connection conn) throws SQLException  ;
    protected abstract int myUpdate(Connection conn) throws SQLException;

    public final Rows execQuery() throws SQLException{
    	Connection conn = getConnection() ;
    	long start = System.currentTimeMillis();
        conn.setTransactionIsolation(Connection.TRANSACTION_READ_COMMITTED);

        // 
    	Rows rows = myQuery(conn) ;

    	long end = System.currentTimeMillis();
    	getDBManager().freeConnection(conn) ;
    	getDBController().handleServant(start, end, this, IQueryable.QUERY_COMMAND);
    	return rows ;
    }
    
    public final int execUpdate() throws SQLException {
        Connection conn = null;
        int result = 0;

        long start = System.currentTimeMillis();
        try {
            conn = getConnection();
            conn.setTransactionIsolation(Connection.TRANSACTION_READ_COMMITTED) ;
            conn.setAutoCommit(false);
            
            result = myUpdate(conn);
            if (conn == null) Debug.line() ;
            conn.commit();
        } catch (NullPointerException ex) {
        	if (conn == null) {
                throw new SQLException("Now first, DBController.initPoolConnection();");
        	}
            conn.rollback();
            throw new SQLException("Null Pointer Exception at execUpdate - " + getExceptionMessage(ex, this));
        } catch (SQLException ex) {
            conn.rollback();
            throw new SQLException(getExceptionMessage(ex, this), ex.getSQLState(), ex.getErrorCode());
        } finally {
            if (conn != null ) {
                conn.setAutoCommit(true);
            }
            getDBManager().freeConnection(conn);
        }
    	long end = System.currentTimeMillis();
    	getDBController().handleServant(start, end, this, IQueryable.UPDATE_COMMAND);

        return result;
    }

    protected boolean isArrivedRequestCancel(){
        return requestCancle  ;
    }
    
    protected final IDBController getDBController() {
    	if (dc == null) throw new NullPointerException("DBController not setted") ;
    	return dc;
    }
    
    private DBManager getDBManager() {
    	return getDBController().getDBManager() ;
    }
    
    private Connection getConnection() throws SQLException{
    	return getDBManager().getConnection() ;
    }
    
    protected final void closeSilence(ResultSet rs, Statement stmt, Connection conn) throws SQLException{
		try { if (rs != null ) rs.close();} catch(SQLException ignore){} 
		try { if (stmt != null ) stmt.close(); } catch(SQLException ignore){}
		// getDBManager().freeConnection(conn);
    }
    
    protected final void closeSilence(Statement stmt) throws SQLException{
		try { if (stmt != null ) stmt.close(); } catch(SQLException ignore){}
    }
    
    
    
    
    
    
    
    
    public String getProcSQL() {
        return procSQL;
    }

	protected String toProcedureSQL() {
		return this.getProcSQL();
	}
    
    public String getProcFullSQL() {
        return getProcSQL();
    }
    protected final int getMaxRows(){
    	return page.getMaxCount() ;
    }
    protected final int getFetchSize() {
    	return page.getListNum() > 100 ? DEFAULT_FETCHSIZE : page.getListNum() ;
        // return limitedRow == 0 ? DEFAULT_FETCHSIZE : limitedRow;
    }

    protected final String getExceptionMessage(Exception ex, Queryable queryable) {
        return ex.getMessage() + "\n\t: " + queryable.toString() + "";
    }

    // this string's format can modify at future
    public String toString() {
        if (this instanceof ParameterQueryable) return ((ParameterQueryable)this).getProcFullSQL() ;
        return getProcSQL();
    }
}
