package com.bleujin.framework.db.procedure;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Types;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang.ArrayUtils;

import com.bleujin.framework.db.IDBController;
import com.bleujin.framework.db.Rows;


/**
 * <p>Title: </p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: </p>
 * @author bleujin
 * @version 1.0
 * use this class for batch job
 */

public class UserCommandBatch extends UserCommand implements IUserCommandBatch{

    private int idx = 0 ;

    protected UserCommandBatch(IDBController dc, String strSQL) {
        super(dc, strSQL, QueryType.USER_COMMAND_BATCH) ;
    }

    public Rows execQuery(int limitRow) {
        throw new java.lang.UnsupportedOperationException("exception.framework.user_command_batch.exec_query.not_supported_method");
    }


    protected Object[][] getParamsAsArray() {
        return (Object[][])params.toArray(new Object[0][]);
    }

    protected Object[] getParam(int index) {
        return (Object[])params.get(index) ;
    }


    protected int myUpdate(Connection conn) throws SQLException{
        int updateCount;
        try {
            pstmt = conn.prepareStatement(getProcSQL());
            pstmt.clearBatch();

            Object[][] values = getParamsAsArray(); // [a][b] a : rows, b : column
            for (int row = 0, rlast = values.length > 0 ? values[0].length : 0 ; row < rlast ; row++) {
                for ( int col = 0, clast = values.length ; col < clast ; col++ ) {
                    Object obj = values[col][row] ;
                    setParam(pstmt, col, obj);
                }
                pstmt.addBatch();
            }
            int[] count = pstmt.executeBatch();
            updateCount = count.length ;

            pstmt.close();
            pstmt = null ;

        }
        catch (SQLException ex) {
            throw ex ;
        } finally{
			closeSilence(pstmt) ;
        }
        return updateCount ;
    }

    public IUserCommandBatch addParam(String[] param){
        addParam(idx++, param);
        return this ;
    }
    public IUserCommandBatch addParam(int[] param){
        addParam(idx++, param);
        return this ;
    }
    public IUserCommandBatch addParam(boolean[] param){
        addParam(idx++, param);
        return this ;
    }
    public IUserCommandBatch addParam(long[] param){
        addParam(idx++, param);
        return this ;
    }
    public IUserCommandBatch addClob(String[] param){
        addClob(idx++, param) ;
        return this ;
    }



    public void addClob(int paramindex, String clobString){
        this.addParam(paramindex, new String[]{clobString}, Types.CLOB);
    }
    public void addClob(int paramindex, String[] clobString){
        this.addParam(paramindex, clobString, Types.CLOB);
    }
    public void addClobSize(int paramindex, String clobString, int size){
        String[] values = new String[size] ;
        for (int i = 0; i < values.length; i++) {
            values[i] = clobString ;
        }
        addParam(paramindex, values, Types.CLOB);
    }

    public void addBlob( int paramindex, String fileName) throws FileNotFoundException {
        addBlobSize(paramindex, fileName, 1) ;
    }
    public void addBlob(int paramindex, InputStream is) {
        addBlobSize(paramindex, is, 1) ;
    }
    public void addBlobSize( int paramindex, String fileName, int size) throws FileNotFoundException {
        InputStream[] values = new FileInputStream[size] ;
        for (int i = 0; i < values.length; i++) {
            values[i] = new FileInputStream(fileName) ;
        }
        addParam(paramindex, values, Types.BLOB) ;
    }
    public void addBlobSize( int paramindex, InputStream is, int size) {
        InputStream[] values = new InputStream[size] ;
        for (int i = 0; i < values.length; i++) {
            values[i] = is ;
        }
        addParam(paramindex, values, Types.BLOB) ;
    }
    public void addBlob( int paramindex, String[] fileName) throws FileNotFoundException {
        InputStream[] values = new FileInputStream[fileName.length] ;
        for (int i = 0; i < values.length; i++) {
            values[i] = new FileInputStream(fileName[i]) ;
        }
        addParam(paramindex, values, Types.BLOB) ;
    }
    public void addBlob(int paramindex, InputStream[] is) {
        addParam(paramindex, is, Types.BLOB);
    }


    public void addParam( int paramindex, int value ) {
        this.addParam(paramindex, new Integer[]{new Integer(value)}, Types.INTEGER);
    }
    public void addParam( int paramindex, boolean value ) {
        this.addParam(paramindex, new Boolean[]{new Boolean(value)}, Types.BOOLEAN);
    }
    public void addParam( int paramindex, long value ) {
        this.addParam(paramindex, new Long[]{new Long(value)}, Types.BIGINT);
    }
    public void addParam( int paramindex, Object value ) {
        this.addParam(paramindex, new Object[]{value}, Types.OTHER);
    }
    public void addParam( int paramindex, String param ) {
        this.addParam(paramindex, new String[]{param}, Types.VARCHAR);
    }
    public void addParam( int paramindex, Object value, int type ) {
        this.addParam(paramindex, new Object[]{value}, type);
    }

    
    
    
    
    public void addParamSize(int paramindex, int value, int size){
        Integer[] values = new Integer[size] ;
        for (int i = 0; i < values.length; i++) {
            values[i] = new Integer(value) ;
        }
        addParam(paramindex, values, Types.INTEGER);
    }
    public void addParamSize(int paramindex, long value, int size){
        Long[] values = new Long[size] ;
        for (int i = 0; i < values.length; i++) {
            values[i] = new Long(value) ;
        }
        addParam(paramindex, values, Types.BIGINT);
    }
    public void addParamSize(int paramindex, boolean value, int size){
        Boolean[] values = new Boolean[size] ;
        for (int i = 0; i < values.length; i++) {
            values[i] = new Boolean(value) ;
        }
        addParam(paramindex, values, Types.BOOLEAN);
    }
    public void addParamSize(int paramindex, String value, int size){
        String[] values = new String[size] ;
        for (int i = 0; i < values.length; i++) {
            values[i] = new String(value) ;
        }
        addParam(paramindex, values, Types.VARCHAR);
    }
    public void addParamSize(int paramindex, Object value, int size){
    	Object[] values = new Object[size] ;
        for (int i = 0; i < values.length; i++) {
            values[i] = value ;
        }
        addParam(paramindex, values, Types.OTHER);
    }
    public void addParamSize(int paramindex, Object value, int size, int type){
    	Object[] values = new Object[size] ;
        for (int i = 0; i < values.length; i++) {
            values[i] = value ;
        }
        addParam(paramindex, values, type);
    }

    
    public void addParamSize(String name, boolean value, int size) {
    	for (int i = 0; i < size; i++) {
			putNamedParam(name, new ParamValue(new Boolean(value), Types.BOOLEAN)) ;
		}
    }
    public void addParamSize(String name, int value, int size) {
    	for (int i = 0; i < size; i++) {
			putNamedParam(name, new ParamValue(new Integer(value), Types.INTEGER)) ;
		}
    }
    public void addParamSize(String name, long value, int size) {
    	for (int i = 0; i < size; i++) {
			putNamedParam(name, new ParamValue(new Long(value), Types.BIGINT)) ;
		}
    }
    public void addParamSize(String name, String value, int size) {
    	for (int i = 0; i < size; i++) {
			putNamedParam(name, new ParamValue(value, Types.VARCHAR)) ;
		}
    }
    public void addParamSize(String name, Object value, int size) {
    	for (int i = 0; i < size; i++) {
			putNamedParam(name, new ParamValue(value, Types.OTHER)) ;
		}
    }
    public void addParamSize(String name, Object value, int size, int type) {
    	for (int i = 0; i < size; i++) {
			putNamedParam(name, new ParamValue(value, type)) ;
		}
    }

    
    
    public void addParam( int paramindex, int[] objs ) {
        addParam(paramindex, ArrayUtils.toObject(objs), Types.INTEGER);
   }
    public void addParam( int paramindex, long[] objs ) {
        addParam(paramindex, ArrayUtils.toObject(objs), Types.BIGINT);
   }
    public void addParam( int paramindex, boolean[] objs){
        addParam(paramindex, ArrayUtils.toObject(objs), Types.BOOLEAN);
    }
    public void addParam( int paramindex, Object[] objs ){
        addParam(paramindex, objs, Types.OTHER);
    }
    public void addParam( int paramindex, String[] strs ){
        addParam(paramindex, strs, Types.VARCHAR);
    }
    public void addParam( int paramindex, Object[] objs, int type ){
        checkArraySize(paramindex, objs.length);
        params.add( paramindex, objs );
        types.add(paramindex, new Integer(type));
    }

    public void addParam( String name, int[] objs ) {
        addParam(name, ArrayUtils.toObject(objs), Types.INTEGER);
   }
    public void addParam( String name, long[] objs ) {
        addParam(name, ArrayUtils.toObject(objs), Types.BIGINT);
   }
    public void addParam( String name, boolean[] objs){
        addParam(name, ArrayUtils.toObject(objs), Types.BOOLEAN);
    }
    public void addParam( String name, Object[] objs ){
        addParam(name, objs, Types.OTHER);
    }
    public void addParam( String name, String[] strs ){
        addParam(name, strs, Types.VARCHAR);
    }
    public void addParam( String name, Object[] objs, int type ){
    	for (int i = 0; i < objs.length; i++) {
    		putNamedParam(name, new ParamValue(objs[i], type)) ;
		}
    }
    
    public void addClob(String name, String[] objs) {
    	for (int i = 0; i < objs.length; i++) {
    		putNamedParam(name, new ParamValue(objs[i], Types.CLOB)) ;
		}
    }
    
    
    
    
    public void addBatchParam(int paramindex, String val){
        addBatchParam(paramindex, val, Types.VARCHAR);
    }
    public void addBatchParam(int paramindex, boolean val){
        addBatchParam(paramindex, val ? Boolean.TRUE : Boolean.FALSE, Types.BOOLEAN);
    }
    public void addBatchParam(int paramindex, int val){
        addBatchParam(paramindex, new Integer(val), Types.INTEGER);
    }
    public void addBatchParam(int paramindex, long val){
        addBatchParam(paramindex, new Long(val), Types.BIGINT);
    }
    public void addBatchParam(int paramindex, Object val){
        addBatchParam(paramindex, val, Types.OTHER);
    }
    public void addBatchClob(int paramindex, String val){
        addBatchParam(paramindex, val, Types.CLOB);
    }
    
    

    public void addBatchParam(String name, String val){
    	putNamedParam(name, new ParamValue(val, Types.VARCHAR)) ;
    }
    public void addBatchParam(String name, boolean val){
    	putNamedParam(name, new ParamValue(new Boolean(val), Types.BOOLEAN)) ;
    }
    public void addBatchParam(String name, int val){
    	putNamedParam(name, new ParamValue(new Integer(val), Types.INTEGER)) ;
    }
    public void addBatchParam(String name, long val){
    	putNamedParam(name, new ParamValue(new Long(val), Types.BIGINT)) ;
    }
    public void addBatchParam(String name, Object val){
    	putNamedParam(name, new ParamValue(val, Types.OTHER)) ;
    }
    public void addBatchParam(String name, Object val, int type) {
    	putNamedParam(name, new ParamValue(val, type)) ;
    }
    public void addBatchClob(String name, String val){
    	putNamedParam(name, new ParamValue(val, Types.CLOB)) ;
    }
    
    private void putNamedParam(String name, ParamValue value){
    	ArrayList<ParamValue> list = (ArrayList<ParamValue>)namedParams.get(name) ;
    	if (list == null){
    		list = new ArrayList<ParamValue>() ;
    	}
    	list.add(value) ;
    	
    	namedParams.put(name, list);
    }
    
    @Override
	protected String transNamedSQL(String _sql) {
		if (namedParams.isEmpty()) {
			return _sql;
		} else {

			String baseSQL = _sql;
			// when select ... from table wheren col1 = :name1 and col2 = :name2
			// and col3 = (:name1 + :name2)
			// list will contain names of: name1, name2, name1, name2
			List list = getSettedParam(baseSQL);
			ArrayList<ParamValue>[] tempParams = new ArrayList[list.size()];

			for (int i = 0, last = list.size(); i < last; i++) {
				ArrayList<ParamValue> paramValues = (ArrayList<ParamValue>) namedParams.get(list.get(i));
				if (paramValues == null) throw new IllegalArgumentException("unsetted named param : " + list.get(i)) ;
				tempParams[i] = paramValues;
			}
			// now, we have tempParam list which contains values for the list.
			
			for (int i = 0; i < tempParams.length; i++) {
				ParamValue[] values = (ParamValue[])tempParams[i].toArray(new ParamValue[0]) ;
				Object[] objs = new Object[values.length];
				for (int j = 0; j < values.length; j++) {
					objs[j] = values[j].value ;
				}
				
				this.params.add(objs);
				this.types.add(tempParams[i].get(0).type);
			}
			return baseSQL.replaceAll(patternStr, "?");
		}
	}

    
    
    public void addBatchParam(final int paramindex, final Object obj, int type){
        if ( params.size() <= paramindex ) {// first value.. is empty
            params.add(paramindex, new Object[]{obj});
            types.add(paramindex, new Integer(type));
        } else {
            Integer beforeType = (Integer)types.get(paramindex);
            if(beforeType.intValue() != type) throw new IllegalArgumentException("Not Match Array Type");

            Object[] src = (Object[])params.get(paramindex);
            Object[] dst = new Object[src.length + 1];
            System.arraycopy(src, 0, dst, 0, Math.min(src.length, dst.length));
            dst[dst.length - 1] = obj;

            params.remove(paramindex) ;
            params.add(paramindex, dst);
        }
    }


    private void checkArraySize(int paramindex, int thisLength){
        if (paramindex == 0)return;
        else {
            Object[] sObjs = (Object[]) params.get(0);
            if (sObjs.length != thisLength) {
                throw new IllegalArgumentException("Not Match Array Size");
            }
        }
    }

    public String toString(){
        StringBuffer result = new StringBuffer("UserCommandBatch : " +getProcSQL() + "\n");

        int paramsize = params.size() ;
        int maxPrintOut = 10 ;


        for(int i=0, last = paramsize > maxPrintOut ? maxPrintOut : paramsize ; i<last ; i++){
            Object[] objects = (Object[])params.get(i);
            if (objects.length > 0) {
                result.append(String.valueOf(i) + " : " + objects[0] + ".....\n") ;
            }
        }
        if (paramsize > maxPrintOut) result.append(".....Total Query Size : " + paramsize) ;

        return result.toString() ;
    }

    public void setParamValues(List params, List types){
        this.params = params ;
        this.types  = types  ;
    }

    protected Statement getStatement() {
        return pstmt;
    }


    public void clearParam() {
    	params.clear() ;
    	types.clear() ;
    }
}
