package com.bleujin.framework.db.procedure;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Types;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang.ArrayUtils;

import com.bleujin.framework.db.IDBController;
import com.bleujin.framework.db.Rows;

public abstract class UserProcedureBatch extends UserProcedure implements IUserProcedureBatch{

    private int idx = 0 ;
    UserProcedureBatch(IDBController dc, String procSQL) {
        super(dc, procSQL, QueryType.USER_PROCEDURE_BATCH) ;
    }
    @Override 
    protected Rows myQuery(Connection conn) throws SQLException {
        throw new java.lang.UnsupportedOperationException("This Method not yet implemented.");
    }


    public void addClob(int paramindex, String clobString){
        this.addParameter(paramindex, new String[]{clobString}, Types.CLOB);
    }
    public void addClob(int paramindex, String[] clobString){
        this.addParameter(paramindex, clobString, Types.CLOB);
    }
    public void addClob(int paramindex, String clobString, int size){
        String[] values = new String[size] ;
        for (int i = 0; i < values.length; i++) {
            values[i] = clobString ;
        }
        addParameter(paramindex, values, Types.CLOB);
    }

    public void addBlob( int paramindex, String fileName) throws FileNotFoundException {
        addBlob(paramindex, fileName, 1) ;
    }
    public void addBlob(int paramindex, InputStream is) {
        addBlob(paramindex, is, 1) ;
    }
    public void addBlob( int paramindex, String fileName, int size) throws FileNotFoundException {
        InputStream[] values = new FileInputStream[size] ;
        for (int i = 0; i < values.length; i++) {
            values[i] = new FileInputStream(fileName) ;
        }
        addParameter(paramindex, values, Types.BLOB) ;
    }
    public void addBlob( int paramindex, InputStream is, int size) {
        InputStream[] values = new InputStream[size] ;
        for (int i = 0; i < values.length; i++) {
            values[i] = is ;
        }
        addParameter(paramindex, values, Types.BLOB) ;
    }
    public void addBlob( int paramindex, String[] fileName) throws FileNotFoundException {
        InputStream[] values = new FileInputStream[fileName.length] ;
        for (int i = 0; i < values.length; i++) {
            values[i] = new FileInputStream(fileName[i]) ;
        }
        addParameter(paramindex, values, Types.BLOB) ;
    }
    public void addBlob(int paramindex, InputStream[] is) {
        addParameter(paramindex, is, Types.BLOB);
    }


    public IUserProcedureBatch addParam(Object param){
        if (param.getClass().isArray()){
            addParam(idx++, (Object[])param);
        } else {
            addParam(idx++, param);
        }
        return this ;
    }
    public IUserProcedureBatch addParam(Object[] param){
        this.addParam(idx++, param);
        return this ;
    }
    public IUserProcedureBatch addParam(Integer[] param){
        this.addParam(idx++, param);
        return this ;
    }
    public IUserProcedureBatch addParam(String param){
        addParam(idx++, param);
        return this ;
    }
    public IUserProcedureBatch addParam(String param[]){
        addParam(idx++, param);
        return this ;
    }
    public IUserProcedureBatch addParam(int param){
        addParam(idx++, param);
        return this ;
    }
    public IUserProcedureBatch addParam(int[] param){
        addParam(idx++, param);
        return this ;
    }
    public IUserProcedureBatch addClob(String param){
        this.addParameter(idx++, new String[]{param}, Types.CLOB);
        return this ;
    }
    public IUserProcedureBatch addClob(String[] param){
        this.addParameter(idx++, param, Types.CLOB);
        return this ;
    }



    public void addParam( int paramindex, int value ) {
        this.addParameter(paramindex, new Integer[]{new Integer(value)}, Types.INTEGER);
    }
    public void addParam( int paramindex, boolean value ) {
        this.addParameter(paramindex, new Boolean[]{new Boolean(value)}, Types.BOOLEAN);
    }
    public void addParam( int paramindex, Object value ) {
        this.addParameter(paramindex, new Object[]{value}, Types.OTHER);
    }
    public void addParam( int paramindex, String param ) {
        this.addParameter(paramindex, new String[]{param}, Types.VARCHAR);
    }
    public void addParamAsString( int paramindex, String param ) {
        this.addParameter(paramindex, new String[]{param}, Types.VARCHAR);
    }
    public void addParam( int paramindex, Object value, int type ) {
        this.addParameter(paramindex, new Object[]{value}, type);
    }



    public void addParam(int paramindex, int value, int size){
        Integer[] values = new Integer[size] ;
        for (int i = 0; i < values.length; i++) {
            values[i] = new Integer(value) ;
        }
        addParameter(paramindex, values, Types.INTEGER);
    }
    public void addParam(int paramindex, boolean value, int size){
        Boolean[] values = new Boolean[size] ;
        for (int i = 0; i < values.length; i++) {
            values[i] = new Boolean(value) ;
        }
        addParameter(paramindex, values, Types.BOOLEAN);
    }
    public void addParam(int paramindex, String value, int size){
        String[] values = new String[size] ;
        for (int i = 0; i < values.length; i++) {
            values[i] = new String(value) ;
        }
        addParameter(paramindex, values, Types.VARCHAR);
    }


    public void addParam( int paramindex, int[] objs ) {
         addParameter(paramindex, ArrayUtils.toObject(objs), Types.INTEGER);
    }
    public void addParam( int paramindex, boolean[] objs){
        addParameter(paramindex, ArrayUtils.toObject(objs), Types.BOOLEAN);
    }
    public void addParam( int paramindex, Object[] objs ){
        addParameter(paramindex, objs, Types.OTHER);
    }
    public void addParam( int paramindex, String[] strs ){
        addParameter(paramindex, strs, Types.VARCHAR);
    }
    public void addParameter( int paramindex, Object[] objs, int type ){
        checkArraySize(paramindex, objs.length);
		if (type == Types.OTHER && objs instanceof IStringObject[]){
			List temp = new ArrayList();
			for (IStringObject obj : (IStringObject[])objs) {
				temp.add(obj.getString()) ;
			}
			addParameter(paramindex, (String[])temp.toArray(new String[0]), Types.VARCHAR) ;
		} else {
			params.add( paramindex, objs );
			types.add(paramindex, new Integer(type));
		}
    }



    public void addBatchParam(int paramindex, String val){
        addBatchParameter(paramindex, val, Types.VARCHAR);
    }
    public void addBatchParam(int paramindex, boolean val){
        addBatchParameter(paramindex, val ? Boolean.TRUE : Boolean.FALSE, Types.BOOLEAN);
    }
    public void addBatchParam(int paramindex, int val){
        addBatchParameter(paramindex, new Integer(val), Types.INTEGER);
    }
    public void addBatchParam(int paramindex, Object val){
        addBatchParameter(paramindex, val, Types.OTHER);
    }

    public void addBatchClob(int paramindex, String val){
        addBatchParameter(paramindex, val, Types.CLOB);
    }

    public void addBatchParameter(final int paramindex, final Object obj, int type){
    	if (type == Types.OTHER && (obj instanceof IStringObject[])){
			List temp = new ArrayList();
			for (IStringObject so : (IStringObject[])obj) {
				temp.add(so.getString()) ;
			}
			addBatchParameter(paramindex, (String[])temp.toArray(new String[0]), Types.VARCHAR) ;
    		return ;
    	}
    	
    	if (type == Types.OTHER && (obj instanceof IStringObject)){
    		addBatchParameter(paramindex, ((IStringObject)obj).getString(), Types.VARCHAR) ;
    		return ;
    	}
    	
        if ( params.size() <= paramindex ) {// first value.. is empty
            params.add(paramindex, new Object[]{obj});
            types.add(paramindex, new Integer(type));
        } else {
            Integer beforeType = (Integer)types.get(paramindex);
            if(beforeType.intValue() != type) throw new IllegalArgumentException("Not Match Array Type. before Type:" + beforeType + ", currentType:" + type);

            Object[] src = (Object[])params.get(paramindex);
            Object[] dst = new Object[src.length + 1];
            System.arraycopy(src, 0, dst, 0, Math.min(src.length, dst.length));
            dst[dst.length - 1] = obj;

            params.remove(paramindex) ;
            params.add(paramindex, dst);
        }
    }


    private void checkArraySize(int paramindex, int thisLength){
        if (paramindex == 0)return;
        else {
            Object[] sObjs = (Object[]) params.get(0);
            if (sObjs.length != thisLength)throw new IllegalArgumentException("Not Match Array Size");
        }
    }

	public String toString(){
        StringBuffer result = new StringBuffer(getProcSQL() + "\n");

        for(int i=0 ; i< params.size() ; i++){
            Object[] objects = (Object[])params.get(i);
            if (objects.length > 0) {
                result.append(String.valueOf(i) + " : " + objects[0] + "..." + objects.length + "\n") ;
            }
        }
        return result.toString() ;
    }

    public void setParamValues(List params, List types){
        this.params = params ;
        this.types  = types  ;
    }

    protected Object[][] getParamsAsArray() {
        return (Object[][])params.toArray(new Object[0][]);
    }

    protected Object[] getParam(int index) {
        return (Object[])params.get(index) ;
    }




}
