package com.bleujin.framework.db.procedure;

import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import com.bleujin.framework.db.IDBController;
import com.bleujin.framework.db.RepositoryException;
import com.bleujin.framework.db.Rows;

public class UserProcedures extends Queryable {

	private List<IQueryable> querys = new ArrayList<IQueryable>();

	private IQueryable currQuery = null;

	public UserProcedures(IDBController dc, String name) {
		super(dc, name, QueryType.USER_PROCEDURES);
	}

	public UserProcedures add(IQueryable query) {
		if (this == query)
			throw new IllegalArgumentException(
					"Cant include self. because of infinite loop");
		if (! (query instanceof Queryable)) throw new IllegalArgumentException(
		"exception.framework.db.userporcedures.not_supported_query_type");
		
		// if (query instanceof XUserProcedure || query instanceof XUserCommand
		// || query instanceof XUserProcedureBatch) throw new
		// IllegalArgumentException("Cant include XUserProcedure.") ;
		querys.add(query);
		return this;
	}

	public UserProcedures add(IQueryable[] queryArray) {
		for (int i = 0, last = queryArray.length; i < last; i++) {
			IQueryable query = queryArray[i];
			add(query);
		}
		return this;
	}

	public int size() {
		return querys.size();
	}

	public Queryable getQuery(int i) {
		return (Queryable) querys.get(i);
	}

	public String getParamAsString(int queryIndex, int paramIndex) {
		return ((ParameterQueryable) getQuery(queryIndex))
				.getParamAsString(paramIndex);
	}

	protected Statement getStatement() {
		throw RepositoryException
				.throwIt("UserProcedures : Not Supported Operation");
	}

	private void setCurrentQuery(IQueryable query) {
		this.currQuery = query;
	}

	protected Rows myQuery(Connection conn) throws SQLException {
		Rows result = null;
		Rows nextRows = null;
		for (int i = 0; i < querys.size(); i++) {
			Queryable query = getQuery(i);
			setCurrentQuery(query);
			if (i == 0) {
				result = query.myQuery(conn);
				nextRows = result;
			} else {
				nextRows = nextRows.setNextRows(query.myQuery(conn));
			}
		}
		return result;
	}

	protected int myUpdate(Connection conn) throws SQLException {
		Queryable query;
		int sumUpdateRow = 0;

		for (int i = 0; i < querys.size(); i++) {
			query = getQuery(i);
			setCurrentQuery(query);
			sumUpdateRow += query.myUpdate(conn);
		}

		return sumUpdateRow;
	}
	
	public List getQuerys(){
		// warn : 
		return Collections.unmodifiableList(querys) ;
	}

	// this string's format can modify at future
	public String toString() {
		StringBuffer result = new StringBuffer("UserProcedures : "
				+ getProcSQL() + "\n");
		int querySize = querys.size();
		int maxPrintOut = 10;

		for (int i = 0, last = querySize > maxPrintOut ? maxPrintOut
				: querySize; i < last; i++) {
			result.append("Query " + i + " : " + getQuery(i).toString() + "\n");
		}
		if (querySize > maxPrintOut)
			result.append("......Total Query Size : " + querySize);
		return result.toString();
	}
}