package com.bleujin.framework.db.querybuilder;

import java.util.ArrayList;

import com.bleujin.framework.util.Converter;


public abstract class SQLExpression
{
    StringBuffer queryString = new StringBuffer(400);
    protected ArrayList filters = new ArrayList() ;
    protected ArrayList columns = new ArrayList() ;
    protected ArrayList orders = new ArrayList() ;
    protected int limit = 0;
    protected int skip = 0 ;
    protected String groupByFields ;
    protected String havingFilters ;

    public SQLExpression()
    {
    }

    public void addColumn(String colName){
        columns.add(colName) ;
    }

    public void setOrder(ArrayList orders){
        this.orders = orders ;
    }
    public void addOrder(String colName){
        orders.add(colName) ;
    }
    public void addOrder(String colName, String order){
        orders.add(colName + " " + order);
    }

    public void setLimit(int limit){
        this.limit = limit ;
    }

    public void setSkip(int skip){
        this.skip = skip;
    }

    public String buildQuery(){
        queryString.setLength(0);
        queryString.append(makeSelect()) ;
        queryString.append(makeColumn()) ;
        queryString.append(makeFrom()) ;
        queryString.append(makeWhere()) ;
        queryString.append(makeGroupby()) ;
        queryString.append(makeHaving()) ;
        queryString.append(makeOrder()) ;
        return queryString.toString() ;
    }

    protected abstract String makeSelect() ;
    protected abstract String makeColumn() ;
    protected abstract String makeFrom() ;
    protected abstract String makeWhere() ;

    public void addGroupBy(String groupByFields)
    {
        this.groupByFields = groupByFields;
    }

    public void addHavingFilter(String havingFilters)
    {
        this.havingFilters = havingFilters;
    }


    protected String makeGroupby(){

        return  this.groupByFields == null ? "" : " group by " + groupByFields;
    }

    protected String makeHaving(){
        return this.havingFilters == null ? "" : " having " + havingFilters;
    }

    protected String makeOrder(){
        String result = "";
        result += makeStringFromArray(orders, ", ", "");
        if (orders.size() > 0){
            result = " order by " + result ;
        }
        return result ;
    }

    public String getQueryString(){
        return buildQuery() ;
    }
    public String toString(){
        return getQueryString() ;
    }

    public void addFilter(String filterString){
        if(filterString.length() > 0 )
            filters.add(filterString) ;
    }

    public void addFilter(String colName, String operator, String colValue){
        filters.add(colName + operator + colValue);
    }

    public void equal(String colName, String colValue){
        addFilter(colName, " = '", colValue + "'") ;
    }
    public void equal(String colName, int colValue){
        addFilter(colName, " = ", String.valueOf(colValue)) ;
    }
    public void notEqual(String colName, String colValue){
        addFilter(colName, " != '", colValue + "'") ;
    }
    public void notEqual(String colName, int colValue){
        addFilter(colName, " != ", String.valueOf(colValue)) ;
    }
    public void anyInclude(String colName, String colValue){
        addFilter(colName, " like '", "%" + colValue + "%'" ) ;
    }
    public void greater(String colName, String colValue){
        addFilter(colName, " > '", colValue + "'" ) ;
    }
    public void greater(String colName, int colValue){
        addFilter(colName, " > ", String.valueOf(colValue )) ;
    }
    public void less(String colName, String colValue){
        addFilter(colName, " < '", colValue + "'" ) ;
    }
    public void less(String colName, int colValue){
        addFilter(colName, " < ", String.valueOf(colValue) ) ;
    }

    String makeStringFromArray(ArrayList array, String divString, String repString)
    {
        return Converter.arrayToStringWithNoEndMark((String[])array.toArray(new String[0]), divString, repString) ;
    }


}