package com.bleujin.framework.util;

import java.util.Arrays;
import java.util.Comparator;

/**
 * @author Kim, Sanghoon (wizest@i-on.net)
 * @version 1.0
 */

public class ComparableParser extends Parser
{
    private final int minStringLength;

    private final Parser[] ps;
    private final int psLen;

    private final Comparator comparator;

    /**
     * @param ps Parser[]
     * @param comparator Comparator  ٸ parser parsing marker  Ѵ. ( ĵǾ  ù° mark õ)
     * @param minStringLength int
     */
    public ComparableParser(Parser[] ps, Comparator comparator, int minStringLength)
    {
        this.ps = ps;
        psLen = ps.length;
        this.comparator = comparator;
        this.minStringLength = minStringLength;
    }

    public void initialize(Marker mark)
    {
        for (int i = 0; i < psLen; ++i) {
            ps[i].initialize(mark);
        }
        super.initialize(mark);
    }

    public void initialize(String text)
    {
        for (int i = 0; i < psLen; ++i) {
            ps[i].initialize(text);
        }
        super.initialize(text);
    }

    public void release()
    {
        for (int i = 0; i < psLen; ++i) {
            ps[i].release();
        }
        super.release();
    }

    private void setAllParsingPoint(int p)
    {
        for (int i = 0; i < psLen; ++i) {
            ps[i].setParsingPoint(p);
        }
    }

    public Marker parseNext() throws ParserException
    {
        setAllParsingPoint(getParsingPoint());

        Marker[] ms = new Marker[psLen];
        for (int i = 0; i < psLen; ++i) {
            ms[i] = ps[i].parseNext();
        }

        Arrays.sort(ms,comparator);

        Marker mark = ms[0];
        setParsingPoint(mark);

        if (mark != null && mark.getValue().length() < minStringLength)
            return parseNext();

        return mark;
    }
}
